/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

 squidview 0.30
 
 A program to nicely browse your squid log
  
 (c) 2001, 2002 Graeme Sheppard

 */


#include <curses.h>
#include <time.h>
#include <string>


#define nInputLen 80

#define nReadBuffer 2048

//#define nUpdateTicks 50   // don't update status line immediately

#define nTimeCol 1      // fields/columns of squid log lines 
#define nCacheHitCol 4
#define nSizeCol 5
#define nTargetCol 7
#define nUserCol_a 8
#define nUserCol_b 3
int iUserCol = nUserCol_a;

#define nUserNameLen 8
#define nSizeCols 10   // width given to display size of requests

typedef long tFilePos;    // positions in log file are of this type 
typedef long tByteTotal;  // for summing bytes transferred

const time_t nSecsInDay = 60 * 60 * 24;

const char cEOL = 10;           // End Of Line character 
const char cCR = 13;
const char cSeperator = 10;

string sHomeDir;
const char szSquidHome[] = ".squidview";
string sPathToFiles;
const char szMainLog[] = "/usr/local/squid/var/logs/access.log";
const char szReadme[] = "/usr/share/doc/squidview/README";

const char szLog1[] = "log1";
const char szLog2[] = "log2";
const char szLog3[] = "log3";
const char szLabel1[] = "Pri";
const char szLabel2[] = "Sec";
const char szLabel3[] = "Tri";
const char* pszCurrentLog;
const char* pszCurrentName;
const char szWordFile[] = "words";         // search words 
const char szUsersFile[] = "users";        // list of real names 
const char szReadmeFile[] = "README";      // link name 

string sLogFile1, sLogFile2, sLogFile3;
string sWordFile, sUsersFile, sSetFileName, sReadmeLink;

string sViewer = "less";
string sCurrentReport = "";
string sCurrentEmail = "";
const string sReportExt = ".txt";

char cDiskBuffer [nReadBuffer +8];     // buffer for log read
char szEmpty = '\0';                   // null string...
char* pcEmpty = &szEmpty;              // ...pointed to by this
char* pcReqBuff = pcEmpty;             // normally the start of the request
                                       // or above if the read op fails

string sWords;
string sStatusMessage = "";

int iSizeHitGrade = 0;
int iSizeHitBytes = 0;
string sSizeHits = "50000 250000 1000000 10000000";

WINDOW* wTemp = 0; // dummy window 

tFilePos iPagePos,        // point in log file of top of current screen 
         iLastPage,       //  " "   "   "  "   of last screen 
         iLogFileSize,
         iLinePos,        // where the current highlighted line is 
         iLastLinePos;    // the last line in log file 

int iMainLines,       // no. of lines in main window 
    iLinesDown,       // which line the highlight bar is at 
    iMaxLinesDown,    // in the case of very small log files 
    iMonitorMode = 1, // like the Unix command tail 
    iCurrentLog = 1;  // which log?

class ct
{
  public:
    ct& operator << (char);
    ct& operator << (bool);
    ct& operator << (const char*);
    ct& operator << (const string&);
    ct& operator << (int);
};

string sDebugText = "";

enum eSearch {hit, miss, veto};
enum eFocus {no_focus, user, cache};
enum eWordHits {no_word, text, CSV};
enum eBandUserTotals {nBUT_none, nBUT_notveto, nBUT_all};
enum eBandDomainTotals {nBDT_none, nBDT_all, nBDT_user};
enum eBandDomainIncludes {nBDI_notveto, nBDI_all};

// log a report options

tFilePos iRepStart = 0,             // report from where
         iRepEnd = 0;               // report to where
int iRepColumns = 80,               // columns of screen?
    iRepCSVcols = 100;              // columns of CSV target
bool bRepSplit = false,             // split long lines
     bRepShowSize = false,          // indent for request size
     bRepKeepExt = true,            // remove innards to show .mp3 etc
     bRepCacheReport = true;
string sRepFileName = "temp.txt",   // report file name
       sRepSeperator = "|",         // for CSV
       sRepFocus = "",
       sRepBDTuser = "",
       sRepTitle = "";
int iRepWordHits = text,
    iRepBUT = nBUT_notveto,
    iRepBDT = nBDT_all,
    iRepBDI = nBDI_notveto,
    iRepFocus = no_focus;
int iRepFast = 10000;
bool bRepMySort = true;

struct sRecordPointer
{
  tByteTotal iBytes;
  int iHits;
  int iRecordNumber;

  sRecordPointer operator= (const sRecordPointer&);
};

struct sListRecords
{
  vector <sRecordPointer> vList;
  
  enum eSortType {nSortHits, nSortBytes};
  
  void Sort (eSortType nSelect);
  void Empty();
};

struct sUserRecord
{
  string sLoginName;
  string sFullName;
};

struct sDomainRecord
{
  string sDomainName;
};


struct rUserTotal
{
  tByteTotal iBytes;
  int iHits;
  string sLoginName;
  string sFullName;

  rUserTotal rUserTotal::operator= (const rUserTotal&);
};

struct rDomainTotal
{
  tByteTotal iBytes;
  int iHits;
  string sDomain;

  rDomainTotal rDomainTotal::operator= (const rDomainTotal&);
};

int iSortFlag = 0;

